// In diesem Beispiel wird der Arduino als Webserver verwendet.
// Eine HTML-Seite wird geschrieben, die die ausgelesenen Daten
// anzeigt und die Dateneingabe per Eingabefeld ermöglicht.

#include <Servo.h>    // Servo-Library importieren
#include <SPI.h>      // SPI-Library importieren (für Serielle Schnittstelle)
#include <Ethernet.h> // Ethernet-Library importieren (für Ethernet-Schnittstelle)

//Hub (Intern) Netzwerkeinstellungen
//byte mac[] = { 0x90, 0xA2, 0xDA, 0x00, 0x82, 0xCC }; // MAC-Adresse des Ethernet-Shields
//byte ip[] = { 192, 168, 14, 10 };                    // IP-Adresse des Arduinos
//byte gateway[] = { 192, 168, 14, 254 };              // Netzwerk-Gateway für das interne Netzwerk
//byte subnet[]  = { 255, 255, 255, 0 };               // Subnetz-Maske für das interne Netzwerk

//ZUHAUS Netzwerkeinstellungen
byte mac[] = { 0x90, 0xA2, 0xDA, 0x00, 0x82, 0xCC }; // MAC-Adresse des Ethernet-Shields
byte ip[] = { 192, 168, 178, 222 };                  // IP-Adresse des Arduinos
byte gateway[] = { 192, 168, 178, 1 };               // Netzwerk-Gateway für das interne Netzwerk
byte subnet[]  = { 255, 255, 255, 0 };               // Subnetz-Maske für das interne Netzwerk

int sensorPin = 0;   // Poti-Pin ("Temperaturmessung")
int servoPin = 9;    // Servo-Pin ("Heizungssteuerung")

int servoRot = 0;    // Variable zum Speichern der Servo-Rotation ("Heizstufe")
int tmpValue = 0;    // Variable zum Speichern der aktuellen "Temperatur"

Server server(80);   // Server-Objekt, um Arduino als Webserver zu verwenden
Servo servo;         // Servo-Objekt zum Steuern des Servos

int msgLength = 100;                 // maximale Länge der zu übertragenden Nachrichten (HTTP-Request)
String readString = String(msgLength);  // Variable zum Speichern von Client-Anfragen (HTTP-Requests)

void setup()
{
  Ethernet.begin(mac, ip, gateway, subnet);  // Ethernet-Verbindung starten
  server.begin();  // Server-Funktionen starten
  
  servo.attach(servoPin);  // Servo am Servo-Pin aktivieren

  Serial.begin(9600); // Serielle Übertragung starten
}

void loop()
{
  Client client = server.available();  // potentiell verbundenen Client speichern
  
  if (client)  // wenn ein Client verbunden ist
  {
    while (client.connected()) // solang der Client verbunden ist
    {
      if (client.available())  // wenn der Client eine Nachricht gesendet hat
      {
        char c = client.read(); // lies das nächste byte der Nachricht ein
  
        if (readString.length() < msgLength) // solange die Nachricht kürzer als maximale Zeichenzahl ist
        {
          readString = readString + c; // hänge das eingelesene Zeichen an readString an
        }

        if (c == '\n')  // wenn Zeilenumbruch erreicht ist (Ende des HTTP-Requests)
        {  
          tmpValue = analogRead(sensorPin); // Temperatur vom Sensor-Pin auslesen und in tmpValue speichern
          
          if(readString.indexOf("Temperatur") > -1) // wenn die eingelesene Nachricht das Wort Temperatur enthält
          {
            // die Nachricht so beschneiden, dass nur noch der übertragene Zahlenwert übrig bleibt und in valueStr speichern
            String valueStr = readString.substring(readString.indexOf("Temperatur")+11, readString.lastIndexOf(" "));

            char charArray [4];                 // ein neues Character-Array anlegen
            valueStr.toCharArray(charArray, 4); // den eingelesenen Zahlenwert in das Array schreiben
            int value = atoi(charArray);        // das Array in einen Integer-Wert umwandeln und in value speichern

            servo.write(value);  // den Servo auf den übertragenen Zahlenwert drehen
          }
          
          tmpValue = analogRead(sensorPin);  // den Temperatur-Wert vom Sensor-Pin auslesen
 
          client.println("HTTP/1.1 200 OK"); // Dokumenttyp-Deklaration für HTML-Datei
          client.println("Content-Type: text/html"); // Dokumenttyp-Deklaration für HTML-Datei
          client.println(); 
          client.print("<html>"); // Start-Tag für HTML-Datei
          client.print("<head>"); // Start-Tag für Header
          client.print("<title>Arduino Webserver</title>");  // Titel der Webseite zwischen Start- und End-Tag
          client.println("</head>"); // End-Tag für Header
          client.print("<body bgcolor='#333333'>"); // Start-Tag für den HTML-Rumpf (Hintergrundfarbe Dunkelgrau)
          client.print("<table align='center'>");  // Start-Tag für Tabelle
          client.print("<tr align='center'><td>"); // Start-Tags für Tabellenzeile (zentrierter Inhalt) und eine Zelle
          client.print("<span style='font-size:48px; font-family:Helvetica, Arial; font-weight:bold; color:#DDDDDD'>"); // Start-Tag für Formatierung des Zelleninhalts (Schriftgröße/-art/-schnitt/-farbe)
          client.print(tmpValue); // Einfügen des ausgelesenen Temperatur-Werts
          client.print("</span>"); // End-Tag der Textformatierung
          client.print("</td></tr>");  // End-Tags der Tabellenzeile und Zelle
          client.print("<tr align='center'><td>"); // Start-Tags für Tabellenzeile (zentrierter Inhalt) und eine Zelle
          client.print("<form method=get>"); // Start-Tag für ein Formular (z.B. für Eingabefelder, Buttons usw.)
          client.print("<input name=Temperatur type=text size=3 maxlength=3 style='font-size:48px; font-family:Helvetica, Arial; text-align:center;'>"); // Eingabefeld mit Name "Temperatur" (max. 3 Zeichen)
          client.print("</td></tr>"); // End-Tags der Tabellenzeile und Zelle
          client.print("<tr align='center'><td>"); // Start-Tags für Tabellenzeile (zentrierter Inhalt) und eine Zelle
          client.print("<input type=submit>"); // Button zum abschicken des Formulars (des Zahlenwerts im Eingabefeld)
          client.print("</form>"); // End-Tag für Formular
          client.print("</td></tr>"); // End-Tags der Tabellenzeile und Zelle
          client.print("</table>"); // End-Tag für Tabelle
          client.println("</body>"); // End-Tag für den HTML-Rumpf
          client.println("</html>"); // End-Tag für die HTML-Datei

          readString=""; // readString-Variable entleeren (für neue Nachrichten vorbereiten)

          client.stop(); // die Client-Verbindung beenden
        }
      }
    }
  }
}
